/*
  **
  ** conf.c
  **
  ** I originaly wrote conf.c as part of the drpoxy package 
  ** thanks to Matthew Pratt and others for their additions. 
  **
  ** Copyright 1999 Jeroen Vreeken (pe1rxq@chello.nl)
  **
  ** This software is licensed under the terms of the GNU General 
  ** Public License (GPL). Please see the file COPYING for details.
  ** 
  **
*/

/**
 * How to add a config option :
 *
 *   1. think twice, there are settings enough
 *
 *   2. add a field to 'struct config' (conf.h) and 'struct config conf'
 *
 *   4. add a entry to the config_params array below, if your
 *      option should be configurable by the config file.
 */ 
#include "conf.h"
#include "motion.h"


struct config conf = {
	DEF_WIDTH,
	DEF_HEIGHT,
	DEF_QUALITY,
	DEF_CHANGES,
	0,
	1,
	0,
	DEF_GAP,
	0,
	IN_DEFAULT,
	0,
	100,
	0,
	VIDEO_DEVICE,
	NULL,
	NULL,
	NULL,
	NULL,
};



static void copy_bool(char *, void *);
static void copy_int(char *, void *);
static void copy_string(char *, void *);


config_param config_params[] = {
	{
	"always_changes",
	&conf.always_diff,
	copy_bool
	},
	{
	"deamon",
	&deamon,
	copy_bool
	},
	{
	"snapshots",
	&alarmtime,
	copy_int
	},
	{
	"threshold",
	&conf.max_changes,
	copy_int
	},
	{
	"framerate",
	&conf.frame_limit,
	copy_int
	},
	{
	"locate",
	&conf.locate,
	copy_bool
	},
	{
	"output_motion",
	&conf.motion_img,
	copy_bool
	},
	{
	"output_normal",
	&conf.new_img,
	copy_bool
	},
	{
	"target_dir",
	&conf.filepath,
	copy_string
	},
	{
	"videodevice",
	&conf.device,
	copy_string
	},
	{
	"input",
	&conf.input,
	copy_int
	},
	{
	"norm",
	&conf.norm,
	copy_int
	},
	{
	"quality",
	&conf.quality,
	copy_int
	},
	{
	"width",
	&conf.width,
	copy_int
	},
	{
	"height",
	&conf.height,
	copy_int
	},
	{
	"execute",
	&conf.externcommand,
	copy_string
	},
	{
	"mail",
	&conf.mail_address,
	copy_string
	},
	{
	"sms",
	&conf.sms_nr,
	copy_string
	},
	{
	"gap",
	&conf.gap,
	copy_int
	},
	{
	"mpeg_encode",
	&conf.mpeg,
	copy_bool
	},
  { NULL, NULL , NULL }
};


void conf_cmdline (int argc, char **argv)
{
	int c;

        while ((c=getopt(argc, argv, "a:BCc:d:E:Df:g:hi:lM:mNn:q:S:s:t:"))!=EOF) 
                switch (c) {                                                    
                        case 'a':                                               
                                sscanf (optarg, "%d", &alarmtime);              
                                break;
			case 'B':
				conf.mpeg=1;                                          
                        case 'C':                                               
                                conf.always_diff=1;                                  
                                break;                                          
                        case 'c':                                               
                                sscanf (optarg, "%d", &conf.max_changes);            
                                break;                                          
                        case 'd':                                               
                                conf.device=optarg;                                  
                                break;                                          
                        case 'E':                                               
                                conf.externcommand=optarg;                           
                                break;                                          
                        case 'D':                                               
                                deamon=1;                                       
                                break;                                          
                        case 'f':                                               
                                sscanf (optarg, "%d", &conf.frame_limit);            
                                break;                                          
                        case 'g':                                               
                                sscanf (optarg, "%d", &conf.gap);                    
                                break;                                          
                        case 'i':                                               
                                sscanf (optarg, "%d", &conf.input);                  
                                break;                                          
                        case 'l':                                               
                                conf.locate=1;                                       
                                break;                                          
                        case 'M':                                               
                                conf.mail_address=optarg;                            
                                break;                                          
                        case 'm':                                               
                                conf.motion_img=1;                                   
                                break;                                          
                        case 'N':                                               
                                conf.new_img=0;                                      
                                break;                                          
                        case 'n':                                               
                                if (!strcmp(optarg, "pal")) conf.norm=NORM_PAL;      
                                if (!strcmp(optarg, "ntsc")) conf.norm=NORM_NTSC;    
                                if (!strcmp(optarg, "secam")) conf.norm=NORM_SECAM;  
                                break;                                          
                        case 'S':                                               
                                conf.sms_nr=optarg;                                  
                                break;                                          
                        case 'q':                                               
                                sscanf (optarg, "%d", &conf.quality);                
                                break;                                          
                        case 's':                                               
                                sscanf (optarg, "%dx%d", &conf.width, &conf.height);      
                                break;                                          
                        case 't':                                               
                                conf.filepath=optarg;                                
                                break;                                          
                        case 'h':                                               
                        /* Do the same as an invallid option, but without the   
                         * 'invalid option' error.... */                        
                        default:                                                
                                usage();                                        
                                exit(-1);                                       
                }                        
}

/**************************************************************************
    Main function, called from motion.c
*/
int conf_load (int argc, char **argv)
{
	FILE *fp;
	char line[256], *cmd = NULL, *arg1 = NULL;
  
	fp = fopen ("motion.conf", "r");
	if (!fp) {	/* specified file does not exist... try default file */
		fp = fopen ("/etc/motion.conf", "r");
		if (!fp) {	/* there is no config file.... use defaults */
			perror("no config file");
		}
	} 
	if (fp) {
		while (fgets(line, 256, fp)) {
			if (!(line[0]=='#')) {	/* skip lines with comment */
				if(strlen(line))
					line[strlen(line) - 1] = 0; /*kill'\n'*/
				else
					line[0]=0;
				cmd = strtok( line, " =" );
				arg1 = strtok( NULL, " =");
				conf_cmdparse(cmd, arg1);
			}
		}
		fclose(fp);
	}
	
	conf_cmdline(argc, argv);
	
	return 0;
}
/*****************************************************************************/
void conf_cmdparse(char *cmd, char *arg1)
{
	int i = 0;

	if( !cmd )return;
	if( !arg1 )return;
  
	while( config_params[i].param_name != NULL ) {
		if(!strncasecmp(cmd, config_params[i].param_name , 255 + 50)) {
			config_params[i].copy( arg1, config_params[i].conf_value );
			return;
		}
		i++;
        }

	fprintf( stderr, "Unkown config option: \"%s\"\n", cmd ); 
        return;
}

/************************************************************************
 * copy functions 
 *
 *   copy_bool   - convert a bool representation to int
 *   copy_int    - convert a string to int
 *   copy_string - just a string copy
 *   
 * @param str -   A char *, pointing to a string representation of the
 *                value.
 * @param value - points to the place where to store the value. 
************************************************************************/
static void copy_bool (char *str, void * val)
{
	if ( !strcmp(str, "1") || 
	     !strcasecmp(str, "yes") || 
	     !strcasecmp(str,"on")) 
	{
		*((int *)val) = 1;
	} else {
		*((int *)val) = 0;
	}
}
static void copy_int(char *str, void * val) {
	*((int *)val) = atoi(str);
}
static void copy_string(char *str, void * val) {
	*(char **)val=malloc(255);
	strncpy(*(char **)val, str, 255);	
}
